/*
# Copyright (c) 2003-2004 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
# ----------------------------------------------------------------------------
#
# Primary author: Purushotham Kamath
#                 Sept  2004
#This material is based upon work supported by the Defense Advanced Research 
#Projects Agency under contract no. N66001-02-1-8939 issued by the Space and 
#Naval Warfare Systems Center (SPAWAR).  Any opinions, findings, and conclusions 
#or recommendations expressed in this material are those of the author(s) and 
#do not necessarily reflect the views of the Defense Advanced Research Projects
#Agency, SPAWAR, or the U.S. Government.
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include <defs.h>
#include <packet.h>
#include <eventhandler.h>
#include <squeue.h>
#include <queue.h>
#include <scheduler.h>
#include <state.h>
#include <rng.h>
#include <trgen.h>
#include <trdump.h>
#include <intransit.h>
#include <codeset.h>
#include <debug.h>
#include <receiver.h>
#include <state.h>


extern Scheduler * sched;
extern InTransit * it;
extern char outputfile[20];
extern Codeset *codeset;
extern Receiver *rcvr;
extern TrGen *trgen;
extern int propSpeed;

TrDump::TrDump():EventHandler()
{
	fout = fopen(outputfile, "w");
	if (fout == NULL)
	{
		dtprintf("trgen.cc: TrDump::TrDump(): Could not open output file : %s for writing\n", outputfile);
		exit(-1);
	}
	totalpkts = 0;
	errorpkts = 0;
	totalonline =0;
	totalsections =0;
	debug = 0;
}

TrDump::~TrDump()
{
	fclose(fout);
}


void TrDump::process(Packet *p)
{

	//Schedule the event for whenever it passes the headend
	dtprintf("trdump.cc: TrDump::process(): Scheduling a packet\n"); 		
	
	 	
	//Distance to headend measured in meters
	double transitdistance = p->depx - 0;
	double transittime = transitdistance/propSpeed;		
	double packetlength = (double)p->size * 8.0 * codeset->N;
	double transmissiontime = packetlength/p->chiprate;
	double totaltime = transittime + transmissiontime;
	double tailtime = p->departuretime+ totaltime;
	double delay = tailtime - sched->currenttime;

		
	dtprintf("trdump.cc: TrDump::process(): Packet id: %d Delay: %.12f s\n", p->id, delay); 		
	dprintf("\n");
	sched->scheduleevent(this, p, delay);	

}




//The event handling function of the TrDump. It dequeues a packet and
//sends it to the next component.
void TrDump::handleevent(Packet *rp)
{
	dtprintf("trdump.cc: TrDump::handleevent(): Entered function\n");
	dtprintf("trdump.cc: TrDump::handleevent(): Receiving packet: %d\n", rp->id);
	dtprintf("trdump.cc: TrDump::handleevent(): Current time: %.12f\n", sched->currenttime);
	


		
	PacketState *pstate = new PacketState(rp);
	pstate->ConstructStateSections(rp);

	dtprintf("trdump.cc: TrDump::handleevent(): Checking each section\n"); 		
	rp->status = PACKET_OK;
	Section * s = pstate->head;

	while(s != NULL)
	{
		int numonline = s->count();
		totalsections++;
		totalonline+=numonline;

		dtprintf("trdump.cc: TrDump::handleevent(): Checking new section\n"); 		
		s->dump();
		rcvr->receive(s, rp);
		if(rp->status == PACKET_ERROR_MUI)
			break;

		s=s->next;
	}		
	



	dtprintf("trdump.cc: TrDump::handleevent():Packet checked Sections: %.6f\n", totalsections); 		
	dprintf("\n");



	if(rp->status == PACKET_ERROR_MUI)
	{
		errorpkts++;
		dtprintf("trgen.cc: TrDump::handleevent(): Packet departing : Error\n");
	}
	else
	{
		dtprintf("trgen.cc: TrDump::handleevent(): Packet departing : Received correctly\n");
	}
	totalpkts++;	
	write(rp);


	delete(rp);
	delete(pstate);
}

void TrDump::write(Packet *p)
{
	fprintf(fout,  "%.12f %d %d.%d %d.%d %d %d\n", sched->currenttime,  p->size,  p->srcaddr, p->srcport, p->dstaddr, p->dstport, p->id, p->status);

}


void TrDump::dumpstats()
{
	normthput = (double)(totalpkts - errorpkts)/(double)(trgen->totalpkts);
	//printf("totalpkts: %d\n", trgen->totalpkts);
	double avgonline = totalonline/totalsections;
	dtprintf("%.2f\n", avgonline);

	double avgsections = (double)totalsections/(double)totalpkts;
	dtprintf("%.2f\n", avgsections);
	printf("%d %d %.2f %.2f %.2f\n", totalpkts, totalpkts - errorpkts, offeredload, offeredload * normthput, avgonline);

}



